/* -*- Mode: vala; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 8 -*- */
/*
 * Copyright (C) 2011 Alexander Larsson <alexl@redhat.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using Gtk;
using Folks;
using Granite;
using Granite.Services;

public class Contacts.App : Granite.Application {
  public Window window;

  private Store contacts_store;
  public ListPane list_pane;
  private ContactPane contacts_pane;
  private Granite.Widgets.Welcome welcome;
  
  private Granite.Widgets.HCollapsablePaned grid;
  public Button action_button;
  private Gtk.MenuItem add_remove_menu_item;
  
  public bool editing = false;
  
  construct {
    program_name = "Dexter";
    exec_name = "dexter";
    app_years = "2011";
    app_icon = "office-address-book";
    app_launcher = "dexter.desktop";
    application_id = "net.launchpad.dexter-rolodex";
    main_url = "https://launchpad.net/dexter-rolodex";
    bug_url = "https://bugs.launchpad.net/dexter-rolodex";
    help_url = "https://answers.launchpad.net/dexter-rolodex";
    translate_url = "https://translations.launchpad.net/dexter-rolodex";
    about_authors = {"Mario Guerriero <mefrio.g@gmail.com>", null };
    about_translators = "Launchpad Translators";
    about_license_type = License.GPL_3_0;
  }
  
  private bool window_delete_event (Gdk.EventAny event) {
    // Clear the contacts so any changed information is stored
    contacts_pane.show_contact (null);
    return false;
  }

  private bool window_map_event (Gdk.EventAny event) {
    list_pane.filter_entry.grab_focus ();
    return true;
  }

  private bool window_key_press_event (Gdk.EventKey event) {
    if ((event.keyval == Gdk.keyval_from_name ("q")) &&
    ((event.state & Gdk.ModifierType.CONTROL_MASK) != 0)) {
      // Clear the contacts so any changed information is stored
      contacts_pane.show_contact (null);
      window.destroy ();
    } else if (((event.keyval == Gdk.keyval_from_name ("s")) ||
        (event.keyval == Gdk.keyval_from_name ("f"))) &&
            ((event.state & Gdk.ModifierType.CONTROL_MASK) != 0)) {
      list_pane.filter_entry.grab_focus ();
    }

    return false;
  }

  private void selection_changed (Contact? new_selection) {
    contacts_pane.show_contact (new_selection);
  }

  public async void show_individual (string id) {
    var contact = yield contacts_store.find_contact ( (c) => {
    return c.individual.id == id;
      });
    if (contact != null) {
      list_pane.select_contact (contact);
      contacts_pane.show_contact (contact);
    } else {
      var dialog = new MessageDialog (/*App.app*/this.window, DialogFlags.DESTROY_WITH_PARENT, MessageType.ERROR, ButtonsType.CLOSE,
                      _("No contact with id %s found"), id);
      dialog.set_title(_("Contact not found"));
      dialog.show ();
      dialog.response.connect ( (id) => {
      dialog.destroy ();
    });
    }
  }

  public async void show_by_email (string email_address) {
    var contact = yield contacts_store.find_contact ( (c) => {
    return c.has_email (email_address);
      });
    if (contact != null) {
      list_pane.select_contact (contact);
      contacts_pane.show_contact (contact);
    } else {
      var dialog = new MessageDialog (/*App.app*/this.window, DialogFlags.DESTROY_WITH_PARENT, MessageType.ERROR, ButtonsType.CLOSE,
                      _("No contact with email address %s found"), email_address);
      dialog.set_title(_("Contact not found"));
      dialog.show ();
      dialog.response.connect ( (id) => {
      dialog.destroy ();
    });
    }
  }

  private void create_window () {
    try {
      var provider = new CssProvider ();
      provider.load_from_path (Config.PKGDATADIR + "/" + "dexter-contacts.css");
      StyleContext.add_provider_for_screen (Gdk.Screen.get_default (), provider,
                        STYLE_PROVIDER_PRIORITY_APPLICATION);
    } catch {
      warning ("Failed to load custom CSS");
    }
    
    //this.app = this;
    window = new Window ();
    window.set_application (this);
    window.set_title ("Dexter");
    window.set_size_request (570, 430);
    window.delete_event.connect (window_delete_event);
    window.map_event.connect (window_map_event);
    window.key_press_event.connect (window_key_press_event);
    
    contacts_pane = new ContactPane (contacts_store);
    contacts_pane.set_hexpand (true);
    
    list_pane = new ListPane (contacts_store, this);
    list_pane.selection_changed.connect (selection_changed);
    list_pane.create_new.connect ( () => {
    contacts_pane.new_contact (list_pane);
      });

    //TOOLBAR
    var toolbar = new Toolbar ();
    toolbar.get_style_context ().add_class (STYLE_CLASS_PRIMARY_TOOLBAR);
    toolbar.set_icon_size (IconSize.LARGE_TOOLBAR);
    toolbar.set_vexpand (false);
    toolbar.set_hexpand (true);
    
    this.action_button = new Button.with_label (_("Edit"));
    //action_button.set_padding (20);
    //action_button.set_size_request (10, 0);
    action_button.clicked.connect (on_action_clicked);
    var action_item = new ToolItem ();
    action_item.add (action_button);
    toolbar.add (action_item);
 
    var spacer = new ToolItem ();
    spacer.set_expand (true);
    toolbar.add (spacer);
    
    list_pane.filter_entry = new Granite.Widgets.SearchBar (_("Search Contacts"));
    list_pane.filter_entry.changed.connect (list_pane.filter_entry_changed);
    list_pane.filter_entry.icon_press.connect (list_pane.filter_entry_clear);

    var search_entry_item = new ToolItem ();
    search_entry_item.is_important = false;
    search_entry_item.add (list_pane.filter_entry);
    toolbar.add (search_entry_item);

    var separator = new SeparatorToolItem ();
    separator.set_draw (false);
    toolbar.add (separator);
    
    var share_button = new Granite.Widgets.ToolButtonWithMenu (new Image.from_icon_name ("document-export", IconSize.LARGE_TOOLBAR), "Share with contractor", new Gtk.Menu ());
    share_button.set_sensitive (false);
    toolbar.add (share_button);
    
    var menu = new Gtk.Menu ();
    add_remove_menu_item = new Gtk.MenuItem.with_label (_("Add/Remove Linked Contacts..."));
    add_remove_menu_item.activate.connect (() => {
      contacts_pane.link_contact ();
    });
    menu.append (add_remove_menu_item);
    var app_menu = (window.get_application() as Granite.Application).create_appmenu(menu);
    toolbar.add (app_menu);
    
    // We make the button slightly wider to look better since it
    // becomes taller when added to the toolbar
    
    toolbar.show_all ();
    
    //toolbar end
    
    welcome = new Granite.Widgets.Welcome (_("Get some friends."), _("Dexter can't seem to find your contacts"));
    welcome.halign = Align.FILL;
    welcome.valign = Align.FILL;
    welcome.append ("address-book-new", _("Import"), _("Add a vCard from your hard dri.ve"));
    welcome.append ("contact-new", _("Create"), _("Create a new contact."));
    welcome.append ("preferences-desktop-online-accounts", _("Sync"), _("Sync Dexter with your online accounts."));
    welcome.set_halign (Align.CENTER);
    welcome.set_valign (Align.CENTER);
    
    var cbox = new Box (Orientation.VERTICAL, 0);
    cbox.pack_start (contacts_pane.notification_area, false, false);
    cbox.pack_start (contacts_pane, true, true);
    cbox.show ();
        
    grid = new Granite.Widgets.HCollapsablePaned ();
    
    grid.add1 (list_pane);
    grid.add2 (cbox);
    
    var container = new Box (Orientation.VERTICAL, 0);
    container.pack_start (toolbar, false, false, 0);

    container.pack_start (grid, true, true, 0);  
    container.pack_start (welcome, true, true, 0);
    debug ("%d", contacts_store.ncontacts);
    
    if (contacts_store.ncontacts == 0) show_welcome ();
    
    welcome.activated.connect ((id) => {
      switch (id) {
        case 0:
            debug ("Not yet implemented");
        break;
            
        case 1:
            hide_welcome ();
            action_add ();
        break;
          
        case 2:
          try {
            Process.spawn_command_line_async ("gnome-control-center online-accounts");
          } catch (Error e) {
              debug ("%s", e.message);
          }
        break;
      }
    });
    
    window.add (container);
    
    container.show ();
    
  }
  
  public void show_welcome () {
    lock_on_welcome ();      
    welcome.show_all ();
    grid.hide ();
    list_pane.hide ();
    contacts_pane.hide ();
  }
  
  public void hide_welcome () {
    unlock_on_welcome ();
    welcome.hide ();
    grid.show ();
    list_pane.show ();
    contacts_pane.show ();
  }
  
  public void lock_actions () {
    action_button.set_sensitive (false);
    list_pane.remove_tb.set_sensitive (false);
    add_remove_menu_item.set_sensitive (false);
  }
  
  public void lock_on_welcome () {
    action_button.hide ();
    list_pane.filter_entry.set_sensitive (false);
    add_remove_menu_item.set_sensitive (false);
  }
  
  public void unlock_on_welcome () {
    action_button.show ();
    action_button.set_sensitive (false);
    list_pane.filter_entry.set_sensitive (true);
    list_pane.remove_tb.set_sensitive (false);
    add_remove_menu_item.set_sensitive (false);
  }
  
  public void unlock_actions () {
    action_button.show ();
    action_button.set_sensitive (true);
    list_pane.remove_tb.set_sensitive (true);
    list_pane.filter_entry.set_sensitive (true);
    add_remove_menu_item.set_sensitive (true);
  }
  
  void on_action_clicked () {
    if (action_button.get_label () == _("Done")) action_save ();
    
    else if (action_button.get_label () == _("Edit")) action_modify ();
    
  }
  
  public void action_add () {
    editing = true;
    list_pane.create_new ();
    action_button.set_label (_("Done"));
  }
  
  public void action_modify () {
    editing = true;
    contacts_pane.display_edit (contacts_pane.selected_contact, null);
    action_button.set_label (_("Done"));
  }
  
  public void action_save () {
    contacts_pane.save_data (); // Ensure all edit data saved
    contacts_pane.display_contact (contacts_pane.selected_contact);
    action_button.set_label (_("Edit"));
    editing = false;
  }
  
  public void action_delete () {
    contacts_pane.show_notification_area ();
    Timeout.add (2500, () => {
      if (!contacts_pane.undo) {
        contacts_pane.delete_contact ();
        action_button.set_sensitive (false);
      
        if (contacts_store.ncontacts == 1) show_welcome ();
        contacts_pane.hide_notification_area ();
      }
      return false;      
    });
  }
  
  public override void startup () {
    ensure_eds_accounts ();
    contacts_store = new Store ();
    base.startup ();
  }

  public override void activate () {
    if (window == null) {
      create_window ();

      // We delay the initial show a tiny bit so most contacts are loaded when we show
      contacts_store.quiescent.connect (() => {
      window.show ();
    });
      // Wait at most 0.5 seconds to show the window
      Timeout.add (500, () => {
      window.show ();
      return false;
    });
    } else {
      window.present ();
    }
  }

  private static string individual_id = null;
  private static string email_address = null;
  private static const OptionEntry[] options = {
    { "individual", 'i', 0, OptionArg.STRING, ref individual_id,
      N_("Show contact with this individual id"), null },
    { "email", 'e', 0, OptionArg.STRING, ref email_address,
      N_("Show contact with this email address"), null },
    { null }
  };

  public override int command_line (ApplicationCommandLine command_line) {
    var args = command_line.get_arguments ();
    unowned string[] _args = args;
    var context = new OptionContext (N_("— contact management"));
    context.add_main_entries (options, Config.GETTEXT_PACKAGE);
    context.set_translation_domain (Config.GETTEXT_PACKAGE);
    context.add_group (Gtk.get_option_group (true));

    individual_id = null;
    email_address = null;

    try {
      context.parse (ref _args);
    } catch (Error e) {
      printerr ("Unable to parse: %s\n", e.message);
      return 1;
    }

    activate ();

    if (individual_id != null)
      show_individual.begin (individual_id);
    if (email_address != null)
      show_by_email.begin (email_address);

    return 0;
  }

  public App () {
    Object (application_id: "org.gnome.Contacts", flags: ApplicationFlags.HANDLES_COMMAND_LINE);
    Granite.Services.Logger.initialize ("Dexter");
    Granite.Services.Logger.DisplayLevel = Granite.Services.LogLevel.DEBUG;
  }
}
